#include "compat.h" // precedes all system includes

#include "bits.h"

// Instantiate the lookup table
const log2<Bits::rep> Bits::IteratorBase::rep_log2;

Bits::Bits(const std::string& x) 
 : length(x.size()*8) {
  size_type cells = rep_cells(length);
  rep* ptr = alloc(cells);
  
  if (cells == 0) return;
  
  std::string::const_iterator j = x.begin();
  for (rep* end = ptr + cells - 1; ptr != end ; ++ptr) {
    rep v = 0;
    for (unsigned int k = 0; k < sizeof(rep); ++k) { // unrolled i hope
      v |= static_cast<rep>(*j++) << (8*k);
    }
    *ptr = v;
  }
  
  rep v = 0;
  for (unsigned int k = 0; j != x.end(); ++k)
    v |= static_cast<rep>(*j++) << (8*k);
  *ptr = v;
}

Bits::operator std::string () const {
  std::string out;
  
  size_type cells = rep_cells(length);
  rep* p = ptr();
  
  if (cells == 0) return out;
  std::string::size_type len = length/8;
  out.reserve(len);
  
  for (rep* end = p + cells - 1; p != end ; ++p) {
    rep v = *p;
    for (unsigned int j = 0; j < sizeof(rep); ++j) { // unrolled i hope
      out.push_back(static_cast<char>(v & 0xff));
      v >>= 8;
    }
  }
  
  rep v = *p;
  while (out.size() != len) {
    out.push_back(static_cast<char>(v & 0xff));
    v >>= 8;
  }
  
  return out;
}

std::ostream& operator << (std::ostream& o, const Bits& b) {
  for (Bits::const_reverse_iterator i = b.rbegin(); i != b.rend(); ++i)
    if (*i) o << "1"; else o << "0";
  return o;
}

#if 0

void cvt(const std::string& x) {
  Bits k(x);
  std::cout << k << std::endl;
  std::string y(k);
  std::cout << y << std::endl;
}

int main() {
  cvt("");
  cvt("1");
  cvt("123");
  cvt("1234");
  cvt("1234567");
  cvt("12345678");
  cvt("123456789");
  cvt("123456789012345");
  cvt("1234567890123456");
  cvt("12345678901234567");
  return 0;
}

#endif
