#include "bits.h"

#define BITS_AVOID_BRANCHES 1

inline Bits::Cell::Cell(rep *p, rep m) : ptr(p), mask(m) { }

inline Bits::Cell::operator bool () const {
  return !!(*ptr & mask);
}

inline Bits::Cell& Bits::Cell::operator = (bool x) {
#ifdef BITS_AVOID_BRANCHES
  // Put val{yes,no} outside the if to convince g++ to use cmovne instead of je
  rep valyes = *ptr | mask;
  rep valno  = *ptr & ~mask;
  *ptr = x?valyes:valno;
#else
  if (x)
    *ptr |= mask;
  else
    *ptr &= ~mask;
#endif
  return *this;
}

inline Bits::Cell& Bits::Cell::operator = (const Bits::Cell& x) {
  return *this = bool(x); 
}

inline void Bits::Cell::flip() {
  *ptr ^= mask;
}

inline bool Bits::Cell::operator == (const Bits::Cell& x) const {
  return bool(*this) == bool(x);
}

inline bool Bits::Cell::operator != (const Bits::Cell& x) const {
  return !(*this == x);
}

inline bool Bits::Cell::operator < (const Bits::Cell& x) const {
  return !bool(*this) && bool(x);
}

inline bool Bits::Cell::operator > (const Bits::Cell& x) const {
  return x < *this;
}

inline bool Bits::Cell::operator <= (const Bits::Cell& x) const {
  return !(x < *this);
}

inline bool Bits::Cell::operator >= (const Bits::Cell& x) const {
  return !(*this < x);
}

inline bool Bits::IteratorBase::operator == (const Bits::IteratorBase& i) const {
  return ptr == i.ptr && mask == i.mask;
}

inline bool Bits::IteratorBase::operator != (const Bits::IteratorBase& i) const {
  return !(*this == i);
}

inline bool Bits::IteratorBase::operator < (const Bits::IteratorBase& i) const {
  return ptr < i.ptr || (ptr == i.ptr && mask < i.mask);
}

inline bool Bits::IteratorBase::operator > (const Bits::IteratorBase& i) const {
  return i < *this;
}

inline bool Bits::IteratorBase::operator <= (const Bits::IteratorBase& i) const {
  return !(i < *this);
}

inline bool Bits::IteratorBase::operator >= (const Bits::IteratorBase& i) const {
  return !(*this < i);
}

inline Bits::IteratorBase::IteratorBase(rep* p, rep m) : ptr(p), mask(m) { }

inline void Bits::IteratorBase::inc() {
  if (!(mask <<= 1)) {
    mask = 1;
    ++ptr;
  }
}

inline void Bits::IteratorBase::dec() {
  if (!(mask >>= 1)) {
    mask = UINTMAX_C(1) << (8*sizeof(Bits::rep)-1);
    --ptr;
  }
}

inline void Bits::IteratorBase::add(Bits::IteratorBase::difference_type i) {
  ptr += i / int(8*sizeof(Bits::rep));
  
  Bits::IteratorBase::difference_type low = i % int(8*sizeof(Bits::rep));
  if (low < 0) {
    --ptr;
    low += int(8*sizeof(Bits::rep));
  }
  
  if (!(mask << low)) {
    ++ptr;
    mask >>= (8*sizeof(Bits::rep) - low);
  } else {
    mask <<= low;
  }
}

inline Bits::IteratorBase::difference_type operator - (const Bits::IteratorBase& x, const Bits::IteratorBase& y) {
  Bits::IteratorBase::difference_type offset_x = Bits::IteratorBase::rep_log2(x.mask);
  Bits::IteratorBase::difference_type offset_y = Bits::IteratorBase::rep_log2(y.mask);
  
  return (offset_x-offset_y) + ((Bits::IteratorBase::difference_type)(8*sizeof(Bits::IteratorBase::my_rep)))*(x.ptr - y.ptr);
}

inline Bits::Iterator::Iterator() : IteratorBase(0, 0) { }

inline Bits::Iterator::Iterator(Bits::rep* p, Bits::rep m) : IteratorBase(p, m) { }

inline Bits::Iterator::reference Bits::Iterator::operator * () const {
  return reference(ptr, mask);
}

inline Bits::Iterator::iterator& Bits::Iterator::operator ++ () {
  inc();
  return *this;
}


inline Bits::Iterator::iterator& Bits::Iterator::operator -- () {
  dec();
  return *this;
}

inline Bits::Iterator::iterator Bits::Iterator::operator ++ (int) {
  iterator out = *this;
  inc();
  return out;
}

inline Bits::Iterator::iterator Bits::Iterator::operator -- (int) {
  iterator out = *this;
  dec();
  return out;
}

inline Bits::Iterator::iterator& Bits::Iterator::operator += (Bits::Iterator::difference_type i) {
  add(i);
  return *this;
}

inline Bits::Iterator::iterator& Bits::Iterator::operator -= (Bits::Iterator::difference_type i) {
  return *this += -i;
}

inline Bits::Iterator::iterator Bits::Iterator::operator + (Bits::Iterator::difference_type i) const {
  iterator out = *this;
  return out += i;
}

inline Bits::Iterator::iterator Bits::Iterator::operator - (Bits::Iterator::difference_type i) const {
  iterator out = *this;
  return out -= i;
}

inline Bits::Iterator::reference Bits::Iterator::operator [] (Bits::Iterator::difference_type i) const {
  return *(*this + i);
}

inline Bits::Iterator::iterator operator + (Bits::Iterator::difference_type i, const Bits::Iterator& x) {
  return x + i;
}

inline Bits::ConstIterator::ConstIterator() : IteratorBase(0, 0) { }

inline Bits::ConstIterator::ConstIterator(Bits::rep* p, Bits::rep m) : IteratorBase(p, m) { }

inline Bits::ConstIterator::ConstIterator(const Bits::Iterator& o) : IteratorBase(o) { }

inline Bits::ConstIterator::const_reference Bits::ConstIterator::operator * () const {
  return Cell(ptr, mask);
}

inline Bits::ConstIterator::const_iterator& Bits::ConstIterator::operator ++ () {
  inc();
  return *this;
}


inline Bits::ConstIterator::const_iterator& Bits::ConstIterator::operator -- () {
  dec();
  return *this;
}

inline Bits::ConstIterator::const_iterator Bits::ConstIterator::operator ++ (int) {
  const_iterator out = *this;
  inc();
  return out;
}

inline Bits::ConstIterator::const_iterator Bits::ConstIterator::operator -- (int) {
  const_iterator out = *this;
  dec();
  return out;
}

inline Bits::ConstIterator::const_iterator& Bits::ConstIterator::operator += (Bits::ConstIterator::difference_type i) {
  add(i);
  return *this;
}

inline Bits::ConstIterator::const_iterator& Bits::ConstIterator::operator -= (Bits::ConstIterator::difference_type i) {
  return *this += -i;
}

inline Bits::ConstIterator::const_iterator Bits::ConstIterator::operator + (Bits::ConstIterator::difference_type i) const {
  const_iterator out = *this;
  return out += i;
}

inline Bits::ConstIterator::const_iterator Bits::ConstIterator::operator - (Bits::ConstIterator::difference_type i) const {
  const_iterator out = *this;
  return out -= i;
}

inline Bits::ConstIterator::const_reference Bits::ConstIterator::operator [] (Bits::ConstIterator::difference_type i) const {
  return *(*this + i);
}

inline Bits::ConstIterator::const_iterator operator + (Bits::ConstIterator::difference_type i, const Bits::ConstIterator& x) {
  return x + i;
}

inline Bits::size_type Bits::rep_cells(Bits::size_type bits) {
  return (bits+8*sizeof(rep)-1) / (8*sizeof(rep));
}

inline Bits::rep* Bits::alloc(Bits::size_type cells) {
  if (cells > 1) {
    return un.buffer = new rep[cells];
  } else {
    return &un.local;
  }
}

inline Bits::rep* Bits::ptr() const {
  return (length > 8*sizeof(rep)) ? un.buffer : &un.local;
}

inline Bits::Bits() : length(0) { 
  un.local = 0;
}

inline Bits::Bits(const Bits& x) : length(x.length) {
  if (length > 8*sizeof(rep)) {
    size_type cells = rep_cells(length);
    un.buffer = new rep[cells];
    memcpy(un.buffer, x.un.buffer, cells*sizeof(rep));
  } else {
    un.local = x.un.local;
  }
}

inline Bits::~Bits() {
  if (length > 8*sizeof(rep))
    delete [] un.buffer;
}

inline Bits::Bits(size_type n, const bool& value) 
 : length(n) {
  size_type cells = rep_cells(length);
  rep* ptr = alloc(cells);
  rep ival = value ? static_cast<rep>(-1) : 0;

  if (cells > 0) {
    for (size_type i = 0; i < cells-1; ++i)
      ptr[i] = ival;

    ptr[cells-1] = ival >> ((-length) % (8*sizeof(rep)));
  } else {
    ptr[0] = 0;
  }
}

// Work around superfluous g++ "right shift count >= width of type" warning
inline Bits::size_type Bits::shift_cap(Bits::size_type shift, Bits::size_type size) {
  if (shift > size) return 0;
  return shift;
}

inline Bits::Bits(uint8_t x) : length(8) {
  un.local = x;
}

inline Bits::Bits(uint16_t x) : length(16) {
  size_type cells = rep_cells(16);
  rep* out = alloc(cells);
  for (size_type i = 0; i < cells; ++i) {
    *out++ = x;
    x >>= shift_cap(sizeof(rep)*8, 16);
  }
}

inline Bits::Bits(uint32_t x) : length(32) {
  size_type cells = rep_cells(32);
  rep* out = alloc(cells);
  for (size_type i = 0; i < cells; ++i) {
    *out++ = x;
    x >>= shift_cap(sizeof(rep)*8, 32);
  }
}

inline Bits::Bits(uint64_t x) : length(64) {
  size_type cells = rep_cells(64);
  rep* out = alloc(cells);
  for (size_type i = 0; i < cells; ++i) {
    *out++ = x;
    x >>= shift_cap(sizeof(rep)*8, 64);
  }
}

inline Bits::operator uint8_t () const {
  if (length != 8) throw InvalidConversion();
  return un.local;
}

inline Bits::operator uint16_t () const {
  if (length != 16) throw InvalidConversion();
  uint16_t out = 0;
  size_type cells = rep_cells(16);
  rep* val = ptr();
  for (size_t i = 0; i < cells; ++i)
    out = (out << shift_cap(sizeof(rep)*8, 16)) | val[i];
  return out;
}

inline Bits::operator uint32_t () const {
  if (length != 32) throw InvalidConversion();
  uint32_t out = 0;
  size_type cells = rep_cells(32);
  rep* val = ptr();
  for (size_t i = 0; i < cells; ++i)
    out = (out << shift_cap(sizeof(rep)*8, 32)) | val[i];
  return out;
}

inline Bits::operator uint64_t () const {
  if (length != 64) throw InvalidConversion();
  uint64_t out = 0;
  size_type cells = rep_cells(64);
  rep* val = ptr();
  for (size_t i = 0; i < cells; ++i)
    out = (out << shift_cap(sizeof(rep)*8, 64)) | val[i];
  return out;
}

inline uint8_t Bits::get_uint8(Bits::size_type i) const {
  if (length < (i+1)*8) throw InvalidConversion();
  size_type off = i / sizeof(rep);
  size_type sel = i % sizeof(rep);
  
  return ptr()[off] >> (sel*8);
}

inline void Bits::set_uint8(Bits::size_type i, uint8_t x) {
  if (length < (i+1)*8) throw InvalidConversion();
  size_type off = i / sizeof(rep);
  size_type sel = i % sizeof(rep);
  
  rep m = 0xff;
  rep b = x;
  m <<= sel*8;
  b <<= sel*8;

  (ptr()[off] &= ~m) |= b;
}

inline void Bits::swap(Bits& x) { 
  std::swap(length, x.length);
  Un tmp = x.un;
  x.un = un;
  un = tmp;
}

inline void Bits::swap(Bits& x, Bits& y) {
  x.swap(y);
}

inline Bits& Bits::operator = (const Bits& x) {
  Bits copy(x);
  swap(copy, *this);
  return *this;
}

inline bool Bits::empty() const {
  return !length;
}

inline Bits::size_type Bits::size() const {
  return length;
}

inline Bits::size_type Bits::capacity() const {
  return length;
}

inline void Bits::resize(size_type n) {
  Bits tmp(n, false);
  swap(*this, tmp);
  *this |= tmp;
}

inline void Bits::clear() {
  Bits null;
  swap(*this, null);
}

inline Bits::reference Bits::at(Bits::size_type i) {
#ifdef BITS_AVOID_BRANCHES
  size_type low = i % (8*sizeof(rep));
  size_type high = i / (8*sizeof(rep));

  rep* c1 = &un.local;
  rep* c2 = un.buffer+high;
  rep* c = (length<=8*sizeof(rep))?c1:c2;

  return reference(c, UINTMAX_C(1) << low);
#else
  if (length > 8* sizeof(rep)) {
    size_type low = i % (8*sizeof(rep));
    size_type high = i / (8*sizeof(rep));
    return reference(un.buffer+high, UINTMAX_C(1) << low);
  } else {
    return reference(&un.local, UINTMAX_C(1) << i);
  }
#endif
}

inline Bits::const_reference Bits::at(Bits::size_type i) const {
#ifdef BITS_AVOID_BRANCHES
  size_type low = i % (8*sizeof(rep));
  size_type high = i / (8*sizeof(rep));

  rep* c1 = &un.local;
  rep* c2 = un.buffer+high;
  rep* c = (length<=8*sizeof(rep))?c1:c2;

  return reference(c, UINTMAX_C(1) << low);
#else
  if (length > 8* sizeof(rep)) {
    size_type low = i % (8*sizeof(rep));
    size_type high = i / (8*sizeof(rep));
    return reference(un.buffer+high, UINTMAX_C(1) << low);
  } else {
    return reference(&un.local, UINTMAX_C(1) << i);
  }
#endif
}

inline Bits::reference Bits::operator [] (Bits::size_type i) {
  return at(i);
}

inline Bits::const_reference Bits::operator [] (Bits::size_type i) const {
  return at(i);
}

inline Bits::iterator Bits::begin() {
  return iterator(ptr(), 1);
}

inline Bits::iterator Bits::end() {
  return begin() + length;
}

inline Bits::const_iterator Bits::begin() const {
  return const_iterator(ptr(), 1);
}

inline Bits::const_iterator Bits::end() const {
  return begin() + length;
}

inline Bits::reverse_iterator Bits::rbegin() {
  return reverse_iterator(end());
}

inline Bits::reverse_iterator Bits::rend() {
  return reverse_iterator(begin());
}

inline Bits::const_reverse_iterator Bits::rbegin() const {
  return const_reverse_iterator(end());
}

inline Bits::const_reverse_iterator Bits::rend() const {
  return const_reverse_iterator(begin());
}

inline Bits::reference Bits::front() {
  return *begin();
}

inline Bits::reference Bits::back() {
  return *(end() - 1);
}

inline Bits::const_reference Bits::front() const {
  return *begin();
}

inline Bits::const_reference Bits::back() const {
  return *(end() - 1);
}

inline bool Bits::operator == (const Bits& x) const {
  if (length != x.length) return false;

  size_type cells = rep_cells(length);
  rep* a = ptr();
  rep* b = x.ptr();
  rep* e;

  for (e = a + cells; a != e; ++a, ++b)
    if (*a != *b) return false;
  return true;
}

inline bool Bits::operator != (const Bits& x) const {
  return !(*this == x);
}

inline bool Bits::operator < (const Bits& x) const {
  size_type ea = rep_cells(length);
  size_type eb = rep_cells(x.length);
  size_type em = (ea>eb)?eb:ea;
  rep* a = ptr();
  rep* b = x.ptr();
  rep* e;

  for (e = a + em; a != e; ++a, ++b) {
    if (*a != *b) {
      if (*a < *b) return true;
      if (*a > *b) return false;
    }
  }

  if (length < x.length) return true;
  return false;
}

inline bool Bits::operator > (const Bits& x) const {
  return x < *this;
}

inline bool Bits::operator <= (const Bits& x) const {
  return !(x < *this);
}

inline bool Bits::operator >= (const Bits& x) const {
  return !(*this < x);
}

inline Bits& Bits::operator |= (const Bits& x) {
  size_type sa = rep_cells(length);
  size_type sb = rep_cells(x.length);
  size_type sc = (sa>sb)?sb:sa;
  rep* a = ptr();
  rep* b = x.ptr();
  rep* e;

  for (e = a + sc; a != e; ++a, ++b)
    *a |= *b;

  if (sc > 0 && length < x.length)
    *--a &= static_cast<rep>(-1) >> ((-length)%(8*sizeof(rep)));
  
  return *this;
}

inline Bits& Bits::operator ^= (const Bits& x) {
  size_type sa = rep_cells(length);
  size_type sb = rep_cells(x.length);
  size_type sc = (sa>sb)?sb:sa;
  rep* a = ptr();
  rep* b = x.ptr();
  rep* e;

  for (e = a + sc; a != e; ++a, ++b)
    *a ^= *b;

  if (sc > 0 && length < x.length)
    *--a &= static_cast<rep>(-1) >> ((-length)%(8*sizeof(rep)));
  
  return *this;
}

inline Bits& Bits::operator &= (const Bits& x) {
  size_type sa = rep_cells(length);
  size_type sb = rep_cells(x.length);
  size_type sc, sz; /* copy, zero */
  rep* a = ptr();
  rep* b = x.ptr();
  rep* e;

  if (sa > sb) {
   sc = sb;
   sz = sa-sb;
  } else {
   sc = sa;
   sz = 0;
  }

  for (e = a + sc; a != e; ++a, ++b)
    *a &= *b;
  for (e = a + sz; a != e; ++a)
    *a = 0;
  
  return *this;
}

inline Bits Bits::operator & (const Bits& x) const {
  Bits copy(*this);
  copy &= x;
  return copy;
}

inline Bits Bits::operator | (const Bits& x) const {
  Bits copy(*this);
  copy |= x;
  return copy;
}

inline Bits Bits::operator ^ (const Bits& x) const {
  Bits copy(*this);
  copy ^= x;
  return copy;
}

inline Bits Bits::operator ~ () const {
  Bits copy(length, true);
  copy ^= *this;
  return copy;
}
