#include "common/compat.h"

#include "bus.h"
#include <assert.h>

Bus::Bus(Interconnect* ic) : Component(ic) {
}

static uint32_t from_hex(char v) {
  if (v >= '0' && v <= '9') return (uint32_t)(v - '0');
  return (uint32_t)(v-'A') + 10;
}

uint32_t Bus::device(const std::string& name) {
  assert (name.size() > 9);
  uint32_t b0 = from_hex(name[0]);
  uint32_t b1 = from_hex(name[1]);
  uint32_t b2 = from_hex(name[2]);
  uint32_t b3 = from_hex(name[3]);
  uint32_t b4 = from_hex(name[4]);
  uint32_t b5 = from_hex(name[5]);
  uint32_t b6 = from_hex(name[6]);
  uint32_t b7 = from_hex(name[7]);
  return (b7 <<  0) | (b6 <<  4) | (b5 <<  8) | (b4 << 12) |
         (b3 << 16) | (b2 << 20) | (b1 << 24) | (b0 << 28);
}

static char to_hex(uint32_t x) {
  if (x < 10) return '0' + x;
  return 'A'+(x-10);
}

std::string Bus::makeName(uint32_t num, const std::string& tail) {
  assert (tail.size() > 0);
  
  char hex[10];
  hex[0] = to_hex((num >> 28) & 0xf);
  hex[1] = to_hex((num >> 24) & 0xf);
  hex[2] = to_hex((num >> 20) & 0xf);
  hex[3] = to_hex((num >> 16) & 0xf);
  hex[4] = to_hex((num >> 12) & 0xf);
  hex[5] = to_hex((num >>  8) & 0xf);
  hex[6] = to_hex((num >>  4) & 0xf);
  hex[7] = to_hex((num >>  0) & 0xf);
  hex[8] = '/';
  hex[9] = 0;
  
  return hex + tail;
}
