#ifndef BUS_H
#define BUS_H

#include "component.h"
#include <queue>

/* A Bus has dynamic master ports. This helper class names then:
 *   XXXXXXXX/yyy/zzz
 * where XXXXXXXX is hexadecimal for the device id.
 *       yyy/zzz is the device control (for example, IR/write)
 *
 * In order to route 'recv's to the masterports, a Bus probably needs
 * a Queue. It is included here.
 */
class Bus : public Component {
  public:
    Bus(Interconnect* ic);
    
  protected:
    typedef std::queue<MasterPort*> Queue;
    Queue queue;
    
    static uint32_t device(const std::string& name);
    static std::string makeName(uint32_t num, const std::string& tail);
    static const int prefix = 9; /* XXXXXXXX/ */
};

#endif
