#ifndef DEBRUIJN_CONSTANT_H
#define DEBRUIJN_CONSTANT_H

#include "integer.h"
#include "parity_constant.h"
#include "mask_constant.h"

/* This is an implementation of F.S.Annexstein's algorithm. See:
 * "Generating De Bruijn Sequences: An Efficient Implementation"
 * in IEEE Transactions on Computers, Vol. 46, No. 2, Feb 1997
 */
 
template <int n, uintmax_t seed>
struct debruijn_constant {
  protected:
    // Build the next smallest constant
    typedef debruijn_constant<n-1, (seed>>2)> base;
    
    // These two alternatives select which sequence will be generated
    static const int toggle0 = (seed&1) ? 1 : -1;
    static const int toggle1 = (seed&2)>>1;
    
    static const int i = (1 << (n-2)) + toggle0;
    static const int k = toggle1;
    
    static const uintmax_t C = parity_constant<n-2, base::value>::value;
    static const uintmax_t Cp = ( C) & mask_constant<base::length>::value;
    static const uintmax_t Cn = (~C) & mask_constant<base::length>::value;
    
    static const uintmax_t out0 =  Cp & mask_constant<i-k>::value;
    static const uintmax_t out1 = (Cn >> (i-1+k)) << (i-k);
    static const uintmax_t out2 = (Cn & mask_constant<i-1+k>::value) << (base::length+1-k-k);
    static const uintmax_t out3 = (Cp >> (i-k)) << (base::length+i-k);
  
  public:
    static const int length = base::length*2;
    static const uintmax_t value = out0 | out1 | out2 | out3;
};

/* Base case */
template <uintmax_t seed>
struct debruijn_constant<3, seed> {
  static const int length = 8;
  static const uintmax_t value = (seed&1)?0x1D:0x17;
};

template <uintmax_t seed>
struct debruijn_constant<2, seed> {
  static const int length = 4;
  static const uintmax_t value = 0x3;
};

template <uintmax_t seed>
struct debruijn_constant<1, seed> {
  static const int length = 2;
  static const uintmax_t value = 0x1;
};

#endif
