#ifndef LOG2_H
#define LOG2_H

#include "integer.h"
#include "log2_constant.h"
#include "debruijn_constant.h"

template <typename rep>
class log2 {
  public:
    log2(); // initializes table
    uint8_t operator () (rep value) const; // computes the result
    
  protected:
    typedef log2_constant<sizeof(rep)*8> log2_c;
    typedef debruijn_constant<log2_c::value, 0> debruijn_c;
    
    static const rep multiplier = debruijn_c::value;
    static const int shift = sizeof(rep)*8 - log2_c::value;
    uint8_t table[sizeof(rep)*8];
};

template <typename rep>
uint8_t log2<rep>::operator() (rep value) const {
  return table[(rep)(value*multiplier) >> shift];
}

template <typename rep>
log2<rep>::log2() {
  for (unsigned int i = 0; i < sizeof(table); ++i)
    table[(rep)(multiplier << i) >> shift] = i;
}

#endif
