#include "compat.h" // precedes all system includes

#include "masterport.h"
#include "slaveport.h"
#include "component.h"
#include "interconnect.h"

#include <typeinfo>
#include <assert.h>

MasterPort::MasterPort(const std::string& name, const Pinout& pinout, Component* owner)
 : name_(name), pinout_(pinout), owner_(owner), slave_(0) {
}

MasterPort::~MasterPort() {
  assert (slave_ == 0);
}

void MasterPort::operator () (const Bits& bits) {
  Interconnect* ic = owner_->interconnect();
  
  assert (owner_->enabled());
  assert (owner_->interconnect()->allowRecv || owner_->id == static_cast<int>(ic->nodes.size())-1);
  
  // Only feed the data up if someone cares to receive it
  if (!slave_ || !slave_->owner()->enabled()) 
    throw ProtocolViolation(owner_, "Attempt to recv on disconnected master");
  
  Interconnect::Node& n = ic->nodes[owner_->id];
  n.out_recv.push_back(RecvRequest());
  RecvRequest& rr = n.out_recv.back();
  
  rr.slave = slave_;
  rr.bits = bits;
}

std::ostream& operator << (std::ostream& o, const MasterPort& mp) {
  o << "MasterPort(" << mp.name() << ") => ";
  if (mp.source()) {
    o << typeid(*mp.source()->owner()).name() << "/" << mp.source()->name();
  } else {
    o << "---";
  }
  return o << ": " << mp.pinout();
}
