#ifndef MASTER_PORT_H
#define MASTER_PORT_H

#include "pinout.h"
#include "bits.h"
#include "integer.h"

#include <iostream>

class Component;
class SlavePort;
class Scheduler;

/* A MasterPort is used to input commands into a component.
 */
class MasterPort {
  public:
    ~MasterPort();
    
    // Information about this port
    const std::string& name() const;
    const Pinout& pinout() const;
    Pinout& pinout();
    
    Component* owner();
    const Component* owner() const;
    
    // Which port is feeding this master
    const SlavePort* source() const;
    SlavePort* source();
    bool connected() const;
    
    void operator () (const Bits& bits = Bits());
    
  protected:
    MasterPort(const std::string& name, const Pinout& pinout, Component* owner);
    
    std::string name_;
    Pinout pinout_;
    Component* owner_;
    SlavePort* slave_;
  
  friend class Interconnect;
  friend class Component;
  
  friend std::ostream& operator << (std::ostream& o, const MasterPort& m);
};

inline const std::string& MasterPort::name() const {
  return name_;
}

inline const Pinout& MasterPort::pinout() const {
  return pinout_;
}

inline Pinout& MasterPort::pinout() {
  return pinout_;
}

inline Component* MasterPort::owner() {
  return owner_;
}

inline const Component* MasterPort::owner() const {
  return owner_;
}

inline const SlavePort* MasterPort::source() const {
  return slave_;
}

inline SlavePort* MasterPort::source() {
  return slave_;
}

inline bool MasterPort::connected() const {
  return source() != 0;
}

#endif
