#include "compat.h" // precedes all system includes

#include "pinout.h"
#include "to_string.h"

#include <assert.h>

Pinout::Pinout(const std::string& name, int pins) : name_(name), pins_(pins) {
  for (int i = 0; i < pins; ++i) {
    std::string name = "PIN" + to_string(i);
    pins_[i].name = name;
    pins_[i].direction = INOUT;
    table_[name] = i;
  }
}

const std::string& Pinout::name() const {
  return name_;
}

int Pinout::pins() const {
  return pins_.size();
}

int Pinout::getPin(const std::string& name) const {
  Table::const_iterator i = table_.find(name);
  if (i == table_.end()) return -1;
  return i->second;
}

const std::string& Pinout::getName(int pin) const {
  assert (pin >= 0 && (Pins::size_type)pin < pins_.size());
  return pins_[pin].name;
}

Pinout::Direction Pinout::getDirection(int pin) const {
  assert (pin >= 0 && (Pins::size_type)pin < pins_.size());
  return pins_[pin].direction;
}

void Pinout::setPin(int pin, const std::string& name, Pinout::Direction dir) {
  assert(pin >= 0 && (Pins::size_type)pin < pins_.size() && table_.find(name) == table_.end());
  
  std::string oldname = pins_[pin].name;
  pins_[pin].name = name;
  pins_[pin].direction = dir;
  table_.erase(oldname);
  table_[name] = pin;
}

std::ostream& operator << (std::ostream& o, const Pinout& po) {
  o << "Pinout(" << po.name() << ") = [";
  for (Pinout::Pins::const_iterator i = po.pins_.begin(); i != po.pins_.end(); ++i) {
    o << " " << i->name << "/";
    switch (i->direction) {
      case Pinout::IN:    o << "I";  break;
      case Pinout::OUT:   o << "O";  break;
      case Pinout::INOUT: o << "IO"; break;
    }
  }
  return o << " ]";
}
