#ifndef PINOUT_H
#define PINOUT_H

#include <string>
#include <map>
#include <vector>
#include <iostream>

class Pinout {
  public:
    Pinout(const std::string& name, int pins = 0);
    const std::string& name() const;
    int pins() const;
    
    enum Direction { IN, OUT, INOUT };
    void setPin(int pin, const std::string& name, Direction dir);
    
    int getPin(const std::string& name) const; // -1 = undefined
    const std::string& getName(int pin) const;
    Direction getDirection(int pin) const;
    
  protected:
    struct Pin {
      std::string name;
      Direction direction;
    };
    typedef std::vector<Pin> Pins;
    typedef std::map<std::string, int> Table;
    
    std::string name_;
    Pins pins_;
    Table table_;
  
  friend std::ostream& operator << (std::ostream&, const Pinout& po);
};

/* Concrete pinouts should use this specialized type.
 * It makes Bits constructed with foo_pinout.pins() much faster.
 */
template <int num_pins>
class FixedPinout : public Pinout {
  public:
    FixedPinout(const std::string& name) : Pinout(name, num_pins) { }
    int pins() const { return num_pins; }
};

#endif
