#include "compat.h" // precedes all system includes

#include "slaveport.h"
#include "masterport.h"
#include "component.h"
#include "interconnect.h"
#include "badconnection.h"

#include <typeinfo>
#include <assert.h>

SlavePort::SlavePort(const std::string& name, const Pinout& pinout, Component* owner, bool optional)
 : name_(name), pinout_(pinout), owner_(owner), optional_(optional), master_(0) {
}

SlavePort::~SlavePort() {
  assert (master_ == 0);
}

void SlavePort::operator () (const Bits& bits) {
  assert (owner_->enabled());
  assert (owner_->interconnect()->allowSend || owner_->id == 0);
  
  if (!master_ || !master_->owner()->enabled())
    throw ProtocolViolation(owner_, "Attempt to send on slave with no path to EndOfChain");
  
  Interconnect* ic = owner_->interconnect();
  Interconnect::Node& n = ic->nodes[owner_->id];
  n.out_send.push_back(SendRequest());
  SendRequest& sr = n.out_send.back();
  
  sr.master = master_;
  sr.bits = bits;
}

std::ostream& operator << (std::ostream& o, const SlavePort& sp) {
  o << "SlavePort(" << sp.name() << ") => ";
  if (sp.destination()) {
    o << typeid(*sp.destination()->owner()).name() << "/" << sp.destination()->name();
  } else {
    o << "---";
  }
  return o << ": " << sp.pinout();
}
