#ifndef SLAVE_PORT_H
#define SLAVE_PORT_H

#include "pinout.h"
#include "bits.h"
#include "integer.h"

#include <iostream>

class Component;
class MasterPort;
class Scheduler;

/* A SlavePort is used by a component to send commands to its slaves.
 */
class SlavePort {
  public:
    ~SlavePort();
    
    // Information about this port
    const std::string& name() const;
    const Pinout& pinout() const;
    Pinout& pinout();
    
    Component* owner();
    const Component* owner() const;
    bool optional() const;
    
    // Which port is being controlled
    const MasterPort* destination() const;
    MasterPort* destination();
    bool connected() const;
    
    void operator () (const Bits& bits = Bits());
    
  protected:
    SlavePort(const std::string& name, const Pinout& pinout, Component* owner, bool optional);
    
    std::string name_;
    Pinout pinout_;
    Component* owner_;
    bool optional_;
    MasterPort* master_;
  
  friend class Interconnect;
  friend class Component;
  
  friend std::ostream& operator << (std::ostream& o, const SlavePort& s);
};

inline const std::string& SlavePort::name() const {
  return name_;
}

inline const Pinout& SlavePort::pinout() const {
  return pinout_;
}

inline Pinout& SlavePort::pinout() {
  return pinout_;
}

inline Component* SlavePort::owner() {
  return owner_;
}

inline const Component* SlavePort::owner() const {
  return owner_;
}

inline bool SlavePort::optional() const {
  return optional_;
}

inline const MasterPort* SlavePort::destination() const {
  return master_;
}

inline MasterPort* SlavePort::destination() {
  return master_;
}

inline bool SlavePort::connected() const {
  return destination() != 0;
}

#endif
