#include "common/compat.h"

#include "pinouts/pinouts.h"
#include "arbiter.h"

Arbiter::Arbiter(Interconnect* ic, bool r)
 : Bus(ic),
   slave(*addSlavePort("slave", shift_pinout)),
   recvs(r) {
}

SendQueue::const_iterator Arbiter::send(SendQueue::const_iterator i, SendQueue::const_iterator end) {
  for (; i != end; ++i) {
    if (i->master == &reset) continue;
    
    if (recvs) queue.push(i->master);
    slave(i->bits);
  }
  return i;
}

void Arbiter::recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end) {
  for (; i != end; ++i) {
    if (queue.empty())
      throw ProtocolViolation(this, "Bus queue exhausted during recv");
    
    MasterPort* m = queue.front();
    queue.pop();
    
    (*m)(i->bits);
  }
}

Arbiter::Client Arbiter::addClient() {
  Client out;
  out.it = addMasterPort("master", shift_pinout); // slave.destination()->pinout());
  return out;
}

void Arbiter::removeClient(Client c) {
  removeMasterPort(c.it);
}
