#ifndef ARBITER_H
#define ARBITER_H

#include "common/bus.h"

/* Replicate access to a slave for many masters.
 * If recv=true, assume each 'send' to slave triggers a matching 'recv'.
 * If recv=false, assume 'recv' never called by slave.
 */
class Arbiter : public Bus {
  public:
    Arbiter(Interconnect* ic, bool recv);
    
    void recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end);
    SendQueue::const_iterator send(SendQueue::const_iterator i, SendQueue::const_iterator end);
    
    struct Client {
      public:
        MasterPort& port();
      
      protected:
        Masters::iterator it;
      
      friend class Arbiter;
    };
    
    Client addClient();
    void removeClient(Client c);
    
    SlavePort& slave;
    
  protected:
    bool recvs;
};

inline MasterPort& Arbiter::Client::port() { return *it; }

#endif
