#include "common/compat.h"

#include "delay.h"
#include "pinouts/pinouts.h"

#ifdef __MINGW32__
#include <windows.h>
#else
#define POSIX_C_SOURCE 199309L
#include <time.h>
#endif

Delay::Delay(Interconnect* ic)
 : Component(ic),
   delay (*addMasterPort("delay",  uint64_pinout)),
   effect(*addSlavePort ("effect", command_pinout))
{
}

SendQueue::const_iterator Delay::send(SendQueue::const_iterator i, SendQueue::const_iterator end) {
  for (; i != end; ++i) {
    if (i->master == &reset) continue;
    
    uint64_t delay = static_cast<uint64_t>(i->bits);
    queue.push(delay);
    effect();
  }
  
  return i;
}

void Delay::recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end) {
  for (; i != end; ++i) {
    // noop
  }
}

void Delay::execute() {
  if (queue.empty())
    throw ProtocolViolation(this, "No pending side-effects in execute");
  
  uint64_t delay = queue.front();
  queue.pop();
  
#ifdef __MINGW32__
  Sleep(delay / 1000);
#else
  struct timespec s;
  s.tv_sec  = delay / UINT64_C(1000000000);
  s.tv_nsec = delay % UINT64_C(1000000000);
  
  while (s.tv_sec >= 0 && s.tv_nsec > 0)
    nanosleep(&s, &s);
#endif
}
