#include "common/compat.h"

#include "end-of-chain.h"
#include "common/to_string.h"
#include "pinouts/pinouts.h"

EndOfChain::EndOfChain(Interconnect* ic) 
 : Component(ic) {
}

void EndOfChain::recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end) {
  if (i != end)
    throw ProtocolViolation(this, "Interconnect should only invoke recv empty!");
}

SendQueue::const_iterator EndOfChain::send(SendQueue::const_iterator i, SendQueue::const_iterator end) {
  Bits poke;
  
  for (; i != end; ++i) {
    if (!i->bits.empty())
      throw ProtocolViolation(this, "EndOfChain should receive only commands (empty bits)");
    
    i->master->source()->owner()->execute();
    
    // This is only allowed at the end of chain!
    // Never call recv during send in any other component!
    (*i->master)(poke);
  }
  
  return i;
}

EndOfChain::Effect EndOfChain::addEffect() {
  Effect out;
  out.it = addMasterPort("effect", command_pinout);
  return out;
}

void EndOfChain::removeEffect(Effect e) {
  removeMasterPort(e.it);
}
