#include "common/compat.h"

#include "head-of-chain.h"
#include "common/to_string.h"
#include "common/interconnect.h"
#include "pinouts/pinouts.h"

#include <assert.h>

HeadOfChain::HeadOfChain(Interconnect* ic) 
 : Component(ic), controlId(0) {
}

void HeadOfChain::recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end) {
  for (; i != end; ++i) {
    Map::iterator j = map.find(i->slave);
    if (j != map.end()) {
      CallbackInfo& cbi = j->second;
      (*cbi.cb)(cbi.user, i->bits);
    } else {
      // Must be a reset response; ignore it
    }
  }
}

SendQueue::const_iterator HeadOfChain::send(SendQueue::const_iterator i, SendQueue::const_iterator end) {
  if (i != end)
    throw ProtocolViolation(this, "Interconnect should only invoke send empty!");
  
  return i;
}

HeadOfChain::Control HeadOfChain::addControlVoid(const Pinout& po, void* user, callback cb) {
  Control out;
  out.it = addSlavePort("Control" + to_string(controlId), po, true);
  ++controlId;
  
  CallbackInfo& cbi = map[&*out.it];
  cbi.cb = cb;
  cbi.user = user;
  
  return out;
}

void HeadOfChain::explode(HeadOfChain* obj, const Bits& b) {
  throw ProtocolViolation(obj, "Unexpected recv: " + to_string(b));
}

HeadOfChain::Control HeadOfChain::addControl(const Pinout& po) {
  return addControl(po, this, &explode);
}

SlavePort& HeadOfChain::addReset() {
  return *addSlavePort("reset", command_pinout);
}

void HeadOfChain::removeControl(Control c) {
  assert (c.port().owner() == this);
  
  Map::iterator i = map.find(&c.port());
  assert (i != map.end());
  map.erase(i);
  
  removeSlavePort(c.it);
}
