#ifndef JTAG_CHAIN_H
#define JTAG_CHAIN_H

#include <vector>
#include <queue>
#include "common/bus.h"

class JTAG_Chain : public Bus {
  public:
    JTAG_Chain(Interconnect* ic, int max_irlen = 64, int max_devices = 8);
    
    // Report detected device IDCODEs (after init has run)
    typedef std::vector<uint32_t> IDCODEs;
    IDCODEs detected() const;
    
    void recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end);
    SendQueue::const_iterator send(SendQueue::const_iterator i, SendQueue::const_iterator end);
    
  public:
    SlavePort& IR_write;
    SlavePort& IR_read_write;
    SlavePort& DR_write;
    SlavePort& DR_read_write;
    
  protected:
    int max_irlen;
    int max_devices;
    int ir_len;
    
    struct Device {
      uint32_t code;
      int ir_len;
      int ir_offset;
      
      Device(JTAG_Chain* owner, uint32_t dev);
    };
    
    typedef std::vector<Device> Devices;
    Devices devices;
    
    /* Chain scanning has two read-write passes */
    void scanW();
    void scanR(RecvQueue::const_iterator& i);
    void configWidths();
};

#endif
