#include "common/compat.h"

#include "lm32-base.h"

// #define DEBUG_LM32_JTAG 1

LM32Base::LM32Base(Interconnect* ic)
 : Component(ic),
   lm32_write     (*addSlavePort("lm32_write",      Pinout("LM32", 11))),
   lm32_read_write(*addSlavePort("lm32_read_write", Pinout("LM32", 11), true)),
   lm32_read      (*addSlavePort("lm32_read",       Pinout("LM32", 11), true)) {
}

void LM32Base::send_command(unsigned int cmd) {
  send_byte(cmd << 4);
}

void LM32Base::send_byte(uint8_t b) {
  send_keyval(0, b);
}

void LM32Base::send_keyval(unsigned int key, uint8_t b) {
  uint16_t val = b;
  val <<= 3;
  val |= (key & 0x7);
  Bits x(val);
  x.resize(11);
  lm32_write(x);
#ifdef DEBUG_LM32_JTAG
  std::cerr << "LM32 W:  " << std::hex << key << ", " << std::hex << (int)b << std::endl;
#endif
}

void LM32Base::sendrecv_keyval(unsigned int key, uint8_t b) {
  uint16_t val = b;
  val <<= 3;
  val |= (key & 0x7);
  Bits x(val);
  x.resize(11);
  
  if (lm32_read_write.connected()) {
    lm32_read_write(x);
  } else {
    lm32_read();
    lm32_write(x);
  }
  
#ifdef DEBUG_LM32_JTAG
  std::cerr << "LM32 RW: " << std::hex << key << ", " << std::hex << (int)b << std::endl;
#endif
}

void LM32Base::read_byte() {
  if (lm32_read.connected()) {
    lm32_read();
  } else {
    Bits x(11, false);
    lm32_read_write(x);
  }
  
#ifdef DEBUG_LM32_JTAG
  std::cerr << "LM32 R (W0)" << std::endl;
#endif
}

void LM32Base::recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end) {
  for (; i != end; ++i) {
    Bits c(i->bits);
    c.resize(16);
    uint16_t v = static_cast<uint16_t>(c);
#ifdef DEBUG_LM32_JTAG
    std::cerr << "LM32 R:  " << std::hex << (v&0x7) << ", " << std::hex << (int)(v>>3) << " IN " << std::endl;
#endif
    recv(v & 0x7, v >> 3);
  }
}
 