#ifndef LM32_BASE_H
#define LM32_BASE_H

#include "common/component.h"

/* Common base-class which formats commands */
class LM32Base : public Component {
  public:
    LM32Base(Interconnect* ic);
    
    void recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end); // dispatches to protected recv
    
    SlavePort& lm32_write;
    
    /* One of these must be connected */
    SlavePort& lm32_read_write;
    SlavePort& lm32_read;
    
  protected:
    /* A raw byte send */
    void send_byte(uint8_t b);
    void read_byte();
    
    /* Command choices */
    static const unsigned int READ_MEMORY      = 1; // <address 4*send_byte> <wait memory_busy> <read_byte>
    static const unsigned int WRITE_MEMORY     = 2; // <address 4*send_byte> <data send_byte> <wait memory_busy>
    static const unsigned int READ_SEQUENTIAL  = 3; // <memory_busy> <read_byte>
    static const unsigned int WRITE_SEQUENTIAL = 4; // <data send_byte> <memory_busy>
    static const unsigned int WRITE_CSR        = 5; // <value 4*send_byte> <csr send_byte> <memory_busy>
    static const unsigned int BREAK            = 6;
    static const unsigned int RESET            = 7;
    void send_command(unsigned int cmd);
    
    /* Key choices */
    static const unsigned int UART_WRITE = 1;
    static const unsigned int UART_READ = 2;
    void send_keyval(unsigned int key, uint8_t value);
    void sendrecv_keyval(unsigned int key, uint8_t value);
    
    /* Status values */
    static const unsigned int MEMORY_BUSY        = 4;
    static const unsigned int UART_WRITE_BLOCKED = 2;
    static const unsigned int UART_READ_READY    = 1;
    
    virtual void recv(unsigned int status, uint8_t value) = 0;
};

#endif
