#ifndef LM32_DROM_H
#define LM32_DROM_H

#include "common/component.h"

class LM32DROM : public Component {
  public:
    LM32DROM(Interconnect* ic);
    
    SendQueue::const_iterator send(SendQueue::const_iterator i, SendQueue::const_iterator end);
    void recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end);
    
    void stop();
    
    MasterPort& register_address;
    MasterPort& register_write;
    MasterPort& register_read;
    
    MasterPort& memory_address;
    MasterPort& memory_write;
    MasterPort& memory_read;
    
    MasterPort& breakpoint_type;
    MasterPort& breakpoint_add;
    MasterPort& breakpoint_del;
    
    MasterPort& run;
    
    SlavePort& echo;
    SlavePort& uart_write;
    SlavePort& uart_read;
    SlavePort& uart_poll;
    SlavePort& jtag_break;
    
    static const int PC = 32;
    static const int SIG = 33;
    static const int SP = 28;
    
  protected:
    static const int gdb_regs = 37; /* a subset of the RW regs */
    static const int RW_regs = 38;
    static const int R_regs = 3;
    static const int W_regs = 9;
    static const int regs = RW_regs + R_regs + W_regs;
    
    static const int EA = 30;
    static const int BA = 31;
    static const int CFG = 40;
    static const int DC = 41;
    static const int BP0 = 42;
    static const int WP0 = 46;
    
    enum State { RUNNING, LOAD_BASE, LOAD_REGS, STOPPED } state;
    bool needStop;
    bool needRead;
    uint32_t registers;
    uint32_t activeMem;
    uint8_t activeReg;
    uint8_t activeType;
    
    unsigned char old[regs*4];
    unsigned char now[regs*4];
    
    void read (uint32_t address, uint32_t len);
    void write(uint32_t address, SendQueue::const_iterator& i, SendQueue::const_iterator end);
    
    uint32_t getReg(int reg);
    void setReg(int reg, uint32_t val);
    
    void addBreak(uint32_t address);
    void delBreak(uint32_t address);
    
    void update();
};

#endif
