#include "common/compat.h"

#include "pinouts/pinouts.h"
#include "lm32-read.h"

#include <assert.h>

LM32Read::LM32Read(Interconnect* ic, bool s)
 : LM32Base(ic), 
   read(*addMasterPort("read", uint8_pinout)),
   poll(*addMasterPort("poll", uint8_pinout)),
   safe(s),
   ready(false),
   polling(false) {
}

SendQueue::const_iterator LM32Read::send(SendQueue::const_iterator i, SendQueue::const_iterator end) {
  for (; i != end; ++i) {
    if (i->master == &reset) { 
      ready = false;
      polling = false;
      while (!report.empty()) report.pop();
    } else if (i->master == &poll) {
      if (!polling) {
        if (report.empty())
          read_byte();
        polling = true;
      }
    } else {
      if (polling)
        throw ProtocolViolation(this, "Cannot read until poll completes!");
      
      if (!ready && safe) {
        read_byte();
        report.push(1);
        return i;
      }
      
      int expect = 1;
      send_keyval(UART_READ, 0);
      
      if (safe) {
        ready = false;
      } else {
        /* We pipeline consequetive reads into each other */
        while (++i != end && i->master == &read) {
          sendrecv_keyval(UART_READ, 0);
          ++expect;
        }
        --i;
      }
      
      read_byte(); /* grab the (last) result */
      report.push(expect);
    }
  }
  
  return i;
}

void LM32Read::recv(unsigned int status, uint8_t value) {
  ready = (status & UART_READ_READY) != 0;
  
  if (!report.empty()) {
    if (--report.front() == 0) {
      report.pop();
    } else {
      if (!ready)
        throw ProtocolViolation(this, "LM32 device driver too slow; use safe mode for reads");
    }
    
    read(Bits(value));
  } else {
    assert (polling);
  }
  
  if (report.empty() && polling) {
    polling = false;
    poll(Bits(static_cast<uint8_t>(ready?1:0)));
  }
}
