#include "common/compat.h"

#include "pinouts/pinouts.h"
#include "lm32-write.h"

LM32Write::LM32Write(Interconnect* ic, bool s)
 : LM32Base(ic), 
   write(*addMasterPort("write", uint8_pinout)),
   safe(s),
   busy(s) {
}

SendQueue::const_iterator LM32Write::send(SendQueue::const_iterator i, SendQueue::const_iterator end) {
  for (; i != end; ++i) {
    if (i->master == &reset) continue;
    
    if (safe) {
      /* In safe mode, we block before each write until !busy */
      if (busy) {
        read_byte();
        return i;
      } else {
        uint8_t byte = static_cast<uint8_t>(i->bits);
        send_keyval(UART_WRITE, byte);
        busy = true;
      }
    } else {
      uint8_t byte = static_cast<uint8_t>(i->bits);
      send_keyval(UART_WRITE, byte);
    }
  }
  return i;
}

void LM32Write::recv(unsigned int status, uint8_t value) {
  busy = (status & UART_WRITE_BLOCKED) != 0;
  
  /* Try to detect buffer collision */
  if (busy && !safe)
    throw ProtocolViolation(this, "LM32 device driver too slow; write using safe mode");
}
