#include "common/compat.h"

#include "map.h"
#include "pinouts/pinouts.h"

Map::Map(Interconnect* ic, int al, bool r)
 : Bus(ic),
   address(*addSlavePort("address", Pinout("address", al))),
   data   (*addSlavePort("data",    shift_pinout)),
   addrLen(al),
   recvs(r) {
}

Map::Register Map::addRegister(uint32_t value) {
  Register out;
  out.it = addMasterPort(makeName(value, "data"), shift_pinout);
  return out;
}

void Map::recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end) {
  for (; i != end; ++i) {
    if (queue.empty())
      throw ProtocolViolation(this, "unexpected result does not match a read");
    if (i->slave == &address)
      throw ProtocolViolation(this, "recv'd address response");
    
    MasterPort* m = queue.front();
    queue.pop();
    
    (*m)(i->bits);
  }
}

SendQueue::const_iterator Map::send(SendQueue::const_iterator i, SendQueue::const_iterator end) {
  for (; i != end; ++i) {
    if (i->master == &reset) continue;
    
    uint32_t dev = device(i->master->name());
    Bits addr(dev);
    addr.resize(addrLen);
    
    address(addr);
    data(i->bits);
    if (recvs) queue.push(i->master);
  }
  return i;
}
