#include "common/compat.h"

#include "reduce.h"
#include "pinouts/pinouts.h"

Reduce::Reduce(Interconnect* ic)
 : Component(ic),
   master_write     (*addMasterPort("master_write",      shift_pinout)),
   master_read_write(*addMasterPort("master_read_write", shift_pinout)),
   slave_write      (*addSlavePort ("slave_write",       shift_pinout, true)),
   slave_read_write (*addSlavePort ("slave_read_write",  shift_pinout, true)) {
}

void Reduce::recv(RecvQueue::const_iterator i, RecvQueue::const_iterator end) {
  for (; i != end; ++i) {
    master_read_write(i->bits);
  }
}

SendQueue::const_iterator Reduce::send(SendQueue::const_iterator i, SendQueue::const_iterator end) {
  for (; i != end; ++i) {
    if (i->master == &reset) {
      value.clear(); // nested devices may have forgot state!
      continue;
    }
    
    // eliminate this write?
    if (i->master == &master_write && i->bits == value) continue;
    value = i->bits;
    
    if (i->master == &master_write)
      slave_write(i->bits);
    else
      slave_read_write(i->bits);
  }
  
  return i;
}
